
#include <cstdint>
#include "CyclometerDisplay.h"
#include "images/BackgroundImage.h"

extern const unsigned char font3x9[];
extern const unsigned char font10x16[];

constexpr uint16_t CyclometerDisplay::PALETTE_RGB565[];

CyclometerDisplay CyclometerDisplay::_instance;

CyclometerDisplay::CyclometerDisplay() : _cyclometer(), _ride_speed_bar(55, 44, 43, 38, 3), _avg_speed_bar(55, 44, 43, 41, 2) {
    
}

void CyclometerDisplay::initialize() {
    for(uint8_t idx=0; idx<16; ++idx) {
        Pokitto::Display::palette[idx] = PALETTE_RGB565[idx];
    }
}

void CyclometerDisplay::update() {
    _cyclometer.update();
    
    // Draw the background image with speedometer scale
    uint8_t tmp = Pokitto::Display::invisiblecolor;
    Pokitto::Display::invisiblecolor = -1;
    Pokitto::Display::drawBitmap(0, 0, BackgroundImage);
    Pokitto::Display::invisiblecolor = tmp;
    
    printRideSpeed();
    printRideTime();
    printRideDistance();
    
    constexpr const uint32_t START_ANGLE = 90;
    
    // Draw average speed as a circular bar
    uint32_t avg_speed = (UNIT_CONV * _cyclometer.averageSpeed()).integer(); // speed in 10^-3 miles/h
    uint32_t avg_angle = (avg_speed < 35000) ? (avg_speed * 315 / 35000) : 315;
    _avg_speed_bar.setStartAngle(START_ANGLE);
    _avg_speed_bar.setEndAngle(START_ANGLE + avg_angle);
    _avg_speed_bar.draw();
    
    // Draw current ride speed on top of the average speed bar
    uint32_t ride_speed = (UNIT_CONV * _cyclometer.rideSpeed()).integer(); // speed in 10^-3 miles/h
    uint32_t ride_angle = (ride_speed < 35000) ? (ride_speed * 315 / 35000) : 315;
    _ride_speed_bar.setStartAngle(START_ANGLE);
    _ride_speed_bar.setEndAngle(START_ANGLE + ride_angle);
    _ride_speed_bar.draw();
}

void CyclometerDisplay::printRideSpeed() {
    uint32_t speed_integer = (UNIT_CONV * _cyclometer.rideSpeed() / 1000).integer();
    uint8_t speed_fraction = (UNIT_CONV * _cyclometer.rideSpeed() / 100).integer() % 10;
    
    uint8_t x = (speed_integer < 10) ? 47 : (speed_integer < 100) ? 37 : 27;
    Pokitto::Display::set_cursor(x, 37);
    Pokitto::Display::setFont(font10x16);
    Pokitto::Display::adjustCharStep = 0;
    Pokitto::Display::setColor(3);
    
    Pokitto::Display::printf("%u $", speed_integer); // '$' is the symbol for mph in the font
    
    Pokitto::Display::set_cursor(56, 43);
    Pokitto::Display::setFont(font3x9);
    Pokitto::Display::adjustCharStep = 1;
    Pokitto::Display::setColor(3);
    
    Pokitto::Display::printf(".%u", speed_fraction);
}

void CyclometerDisplay::printRideTime() {
    uint32_t rideTime = _cyclometer.rideTimeMillis(); // ride time in milliseconds
    uint8_t hours = (rideTime / (3600 * 1000)) % 100;
    uint8_t mins = (rideTime / (60 * 1000)) % 60;
    uint8_t secs = (rideTime / 1000) % 60;
    
    uint8_t x = (hours < 10) ? 44 : 40;
    Pokitto::Display::set_cursor(x, 23);
    Pokitto::Display::setFont(font3x9);
    Pokitto::Display::setColor(1);
    
    Pokitto::Display::printf("%u:%02u:%02u", hours, mins, secs);
}

void CyclometerDisplay::printRideDistance() {
    uint32_t rideDist = _cyclometer.rideDistance(); // distance in meters
    uint32_t dist_integer = (UNIT_CONV * rideDist / 1000).integer();
    uint8_t dist_fraction = (UNIT_CONV * rideDist / 100).integer() % 10;
    
    uint8_t x = 48;
    
    // Change printing position based on how many digits there is in the value
    uint32_t ref = 10;
    while(ref <= dist_integer) {
        x -= 4;
        ref *= 10;
    }
    
    Pokitto::Display::set_cursor(x, 57);
    Pokitto::Display::setFont(font3x9);
    Pokitto::Display::setColor(1);
    
    Pokitto::Display::printf("%u.%u mi", dist_integer, dist_fraction);
}
